package com.smartgwt.sample.client;

import com.google.gwt.core.client.EntryPoint;
import com.isomorphic.datasource.DSRequest;
import com.smartgwt.client.core.KeyIdentifier;
import com.smartgwt.client.data.DSCallback;
import com.smartgwt.client.data.DSResponse;
import com.smartgwt.client.data.DataSource;
import com.smartgwt.client.data.Record;
import com.smartgwt.client.data.RecordList;
import com.smartgwt.client.types.Alignment;
import com.smartgwt.client.types.SelectionStyle;
import com.smartgwt.client.types.SortArrow;
import com.smartgwt.client.util.KeyCallback;
import com.smartgwt.client.util.Page;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.IButton;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.Window;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.events.CloseClickEvent;
import com.smartgwt.client.widgets.events.CloseClickHandler;
import com.smartgwt.client.widgets.form.DynamicForm;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.grid.events.EditFailedEvent;
import com.smartgwt.client.widgets.grid.events.EditFailedHandler;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.VLayout;
import com.smartgwt.client.widgets.layout.VStack;
import com.smartgwt.client.widgets.viewer.DetailViewer;
import com.smartgwt.client.widgets.viewer.DetailViewerField;
import com.smartgwt.sample.client.ConcurrentConflictDialog.UserAction;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class BuiltInDS implements EntryPoint {
    private ListGrid boundList;
    private DynamicForm boundForm;
    private IButton saveBtn;
    private DetailViewer boundViewer;
    private IButton newBtn;

    /**
     * This is the entry point method.
     */
    public void onModuleLoad() {
        KeyIdentifier debugKey = new KeyIdentifier();
        debugKey.setCtrlKey(true);
        debugKey.setKeyName("D");

        Page.registerKey(debugKey, new KeyCallback() {
            public void execute(String keyName) {
                SC.showConsole();
            }
        });


        ListGrid grid = new ListGrid();
        grid.setLeft(20);
        grid.setTop(75);
        grid.setWidth(130);
        grid.setLeaveScrollbarGap(false);
        grid.setShowSortArrow(SortArrow.NONE);
        grid.setCanSort(false);
        grid.setFields(new ListGridField("dsTitle", "Select a DataSource"));
        grid.setData(new ListGridRecord[]{
                new DSRecord("Animals", "animals"),
                new DSRecord("Long Running Edit", "supplyItemLong"),
                new DSRecord("Office Supplies", "supplyItem"),
                new DSRecord("Employees", "employees")}
        );
        grid.setSelectionType(SelectionStyle.SINGLE);
        grid.addRecordClickHandler(new RecordClickHandler() {
            public void onRecordClick(RecordClickEvent event) {
                DSRecord record = (DSRecord) event.getRecord();
                bindComponents(record.getDsName());
            }
        });

        grid.draw();

        VStack vStack = new VStack();
        vStack.setLeft(175);
        vStack.setTop(75);
        vStack.setWidth("70%");
        vStack.setMembersMargin(20);

        Label label = new Label();
        label.setContents("<ul>" +
                "<li>select a datasource from the list at left to bind to these components</li>" +
                "<li>click a record in the grid to view and edit that record in the form</li>" +
                "<li>click <b>New</b> to start editing a new record in the form</li>" +
                "<li>click <b>Save</b> to save changes to a new or edited record in the form</li>" +
                "<li>click <b>Clear</b> to clear all fields in the form</li>" +
                "<li>click <b>Filter</b> to filter (substring match) the grid based on form values</li>" +
                "<li>click <b>Fetch</b> to fetch records (exact match) for the grid based on form values</li>" +
                "<li>double-click a record in the grid to edit inline (press Return, or arrow/tab to another record, to save)</li>" +
                "</ul>");
        vStack.addMember(label);

        boundList = new ListGrid();
        boundList.setHeight(200);
        boundList.setCanEdit(true);

        boundList.addRecordClickHandler(new RecordClickHandler() {
            public void onRecordClick(RecordClickEvent event) {
                Record record = event.getRecord();
                boundForm.editRecord(record);
                saveBtn.enable();
                boundViewer.viewSelectedData(boundList);
            }
        });
        
        // add an error handler to handle edit failed error
        boundList.addEditFailedHandler(new EditFailedHandler() {
			
			@Override
			public void onEditFailed(final EditFailedEvent event) {
				
				if (event.getDsResponse().getStatus() == -74)
				{
					// get some data we're going to use
					final int rowNum = event.getRowNum();
					final Record serverValues = event.getDsResponse().getData()[0];
					final Record editedValues = boundList.getEditedRecord(rowNum);
					final String[] fields = boundList.getDataSource().getFieldNames();
					
					final ListGridRecord oldValues = boundList.getRecord(rowNum);

					// create and display a conflict dialog to allow user see database record, his changed
					// record and choose how to resolve the conflict
					final ConcurrentConflictDialog dlg = new ConcurrentConflictDialog(serverValues, editedValues, fields, new ConcurrentConflictDialogCallback() {
						
						/**
						 * This will be called when user choose a option on the conflict dialog
						 */
						@Override
						public void dialogClosed(UserAction userAction) {

							if (userAction == UserAction.KEEP_EDITING)
							{
								// if user wants to continue editing, start editing on the list record
								boundList.startEditing(rowNum, 0, true);
							}
							else if (userAction == UserAction.DISCARD_CHANGES)
							{
								// if user wants to discard all changes, then we discard changes
								// on the row then cancel editing
								boundList.discardAllEdits(new int[]{rowNum},false);
								boundList.cancelEditing();
							}
							else if (userAction == UserAction.SAVE_ANYWAY)
							{
								// user wants to overwrite the database values
								// first set the old values to the values we're received from the server
								for(int i=0;i<fields.length;i++)
								{
									oldValues.setAttribute(fields[i], 
										serverValues.getAttribute(fields[i]));
								}
								
								// perform save once more, this time the server side should not detect
								// any change as old values will match the database record (unless of 
								// course somebody changed them yet again meanwhile)
								boundList.saveAllEdits(null, new int[]{rowNum});
							}
						}
					});
					
					dlg.show();
				}
			}
		});


        vStack.addMember(boundList);

        boundForm = new DynamicForm();
        boundForm.setNumCols(6);
        boundForm.setAutoFocus(false);
        boundForm.setSuppressValidationErrorCallback(false);
        
        vStack.addMember(boundForm);

        HLayout hLayout = new HLayout(10);
        hLayout.setMembersMargin(10);
        hLayout.setHeight(22);

        saveBtn = new IButton("Save");
    	
        saveBtn.addClickHandler(new ClickHandler() {
        	
            public void onClick(final ClickEvent event) {
				saveForm(null);
            }
        });
        
        hLayout.addMember(saveBtn);

        newBtn = new IButton("New");
        newBtn.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                boundForm.editNewRecord();
                saveBtn.enable();
            }
        });
        hLayout.addMember(newBtn);

        IButton clearBtn = new IButton("Clear");
        clearBtn.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                boundForm.clearValues();
                saveBtn.disable();
            }
        });
        hLayout.addMember(clearBtn);

        IButton filterBtn = new IButton("Filter");
        filterBtn.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                boundList.filterData(boundForm.getValuesAsCriteria());
                saveBtn.disable();
            }
        });
        hLayout.addMember(filterBtn);

        IButton fetchBtn = new IButton("Fetch");
        fetchBtn.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                boundList.fetchData(boundForm.getValuesAsCriteria());
                saveBtn.disable();
            }
        });
        hLayout.addMember(fetchBtn);

        vStack.addMember(hLayout);

        boundViewer = new DetailViewer();
        vStack.addMember(boundViewer);

        vStack.draw();
    }
    
    /**
     * Helper method to save the bound form
     * 
     * @param oldValues the old values of the fields. If this is specified, before the 
     * save request is made, form field old values will be set to these values. This is used
     * when user choose to overwrite the database values with his own, case in which the 
     * database values have to become old values in order to perform a successful save
     * (otherwise save will fail as current old values will be different than the ones in the 
     * database record).
     */
    private void saveForm(Record oldValues)
    {
    	// this is required in order to be able to handle errors on server side
        com.smartgwt.client.data.DSRequest req = new com.smartgwt.client.data.DSRequest();
    	req.setWillHandleError(true);
    	
    	// if we're saved with old value, then set them up in the request
    	if (oldValues != null)
    	{
    		req.setOldValues(oldValues);
    	}
    	
    	// save form data
    	boundForm.saveData(new DSCallback(){
    		
			@Override
			public void execute(DSResponse response, Object rawData,
					com.smartgwt.client.data.DSRequest request) 
			{
				// check if we have a concurrency issue
				if (response.getStatus() == -74)
				{
			    	// set up some data we're going to use later in the dialog
			    	final Record editedValues = boundForm.getValuesAsRecord();
					final String[] fields = boundList.getDataSource().getFieldNames();					
					final Record serverValues = response.getData()[0];
					
					// create a conflict dialog and display it to the user
					final ConcurrentConflictDialog dlg = new ConcurrentConflictDialog(serverValues, editedValues, fields, new ConcurrentConflictDialogCallback() {

						/**
						 * This will be called when user choose a option on the conflict dialog
						 */
						@Override
						public void dialogClosed(UserAction userAction) {

							if (userAction == UserAction.KEEP_EDITING)
							{
								// no action needs to be performed if user wants to keep editing
								return;
							}
							else if (userAction == UserAction.DISCARD_CHANGES)
							{
								// if user wants to discard changes, we cancel editing and
								// clear form values
								boundForm.cancelEditing();
								boundForm.clearValues();
							}
							else if (userAction == UserAction.SAVE_ANYWAY)
							{
								// User wants to overwrite the database values.
								// save the form once more, with the server sent values as old values
								saveForm(serverValues);
							}
						}
					});
					
					dlg.show();
				}
				
				// if operation was successful, then clear form values and disable save button
				if (response.getStatus() == DSResponse.STATUS_SUCCESS)
				{
                    boundForm.clearValues();
                    saveBtn.disable();
                }
			}
    		
    	}, req);    	
    }

    private void bindComponents(String dsName) {
        DataSource ds = DataSource.get(dsName);
        boundList.setDataSource(ds);
        boundViewer.setDataSource(ds);
        boundForm.setDataSource(ds);
        boundList.fetchData();
        newBtn.enable();
        saveBtn.disable();
    }
    
}
